/*
 * alloc.c
 *
 *  Created on: 2021/5/17
 *      Author: ueyudiud
 */

#include "nffs.h"
#include "nffs_node.h"

static n_raw_node_ref* nffsAllocRefBlock(void) {
	n_raw_node_ref* block = nliAllocPage(NFFS_PAGE_SIZE);
	if (block == NULL)
		return NULL;

	n_u32 i;
	for (i = 0; i < NFFS_REF_PER_BLOCK; ++i) {
		block[i]._off = REF_EMPTY_NODE;
		block[i]._next = NULL;
	}
	block[i]._off = REF_LINK_NODE;
	block[i]._next = NULL;

	return block;
}

n_code nffsPreallocRawNodeRefs(n_dev* dev, n_block* block, n_u32 nr) {
	n_raw_node_ref** p = &block->_tail_node;
	n_raw_node_ref* ref = *p;

	nmfLog(NMF_ENTITY_SELF, NMF_DBG_LEVEL_INFO, "Reserving %u refs for block 0x%08x", nr, block->_offset);

	if (ref != NULL && !is_empty_ref(ref))
		ref += 1;

	n_u32 i = nr;
	while (i > 0) {
		if (ref == NULL) {
			ref = *p = nffsAllocRefBlock();
			if (ref == NULL)
				return NMF_FAIL(NMF_NOMEM);
		}
		else if (is_link_ref(ref)) {
			p = &ref->_next;
			ref = *p;
			continue;
		}
		i -= 1;
		ref += 1;
	}

	block->_n_alloc_ref = nr;
	return NMF_SUCC;
}

void nffsFreeRawRefBlock(n_raw_node_ref* block) {
	nliFreePage(block);
}

n_inode* nffsAllocINode(void) {
	return (n_inode*) nliAllocRaw(sizeof(n_inode));
}

void nffsFreeINode(n_inode* node) {
	nliFree(node, sizeof(n_inode));
}

void nffsFreeFullDirent(n_dirent* dirent) {
    nliFree(dirent, sizeof(dirent));
}

n_dirent* nffsAllocFullDirent(n_u32 namesize) {
	return nliAllocRaw(sizeof(n_dirent) + namesize);
}