/*
 * build.c
 *
 *  Created on: 2021/5/10
 *      Author: ueyudiud
 */

#include "nffs.h"
#include "nffs_node.h"

static inline n_inode* nextInodeChain(n_i32* i, n_dev* dev) {
    while((*i) < dev->_inode_capacity) {
        if(dev->_inode_caches[*i])
            return dev->_inode_caches[*i];
        (*i)++;
    }
    return NULL;
}

static inline n_inode* nextInode(n_i32* i, n_inode* in, n_dev* dev) {
    if(in->_next)
        return in->_next;
    (*i)++;
    return nextInodeChain(i, dev);
}

#define forEachInode(i, in, dev)        \
    for (i = 0, in = nextInodeChain(&i, (dev)); in; in = nextInode(&i, in, (dev)))


static void nffsBuildInodePass1(n_dev* dev, n_inode* in, n_i32* dir_hardlinks) {
    // TODO finish this function
}

static n_code buildFileSystem(n_dev* dev) {
	n_code ret;

	void* map_buf = nliAllocRaw(dev->_param._bytes_per_block);
	n_u32 map_buf_len = dev->_param._bytes_per_block;
	if (map_buf == NULL)
		return NMF_FAIL(NMF_NOMEM);

	/* 扫描闪存 */
	for (n_u32 i = 0; i < dev->_param._block_per_total; ++i) {
		n_block* block = &dev->_blocks[i];

		ret = dev->_vtable._read(dev, block->_offset, map_buf, map_buf_len);
		if (nmfIsError(ret))
			goto error;

		ret = nffsScan(dev, block, map_buf);
		if (nmfIsError(ret))
			goto error;
	}

    n_inode* in;
	n_i32 i, dir_hardlinks, dead_fds;
	forEachInode(i, in, dev) {
	    if(in->_scan_dents) {
            nffsBuildInodePass1(dev, in, &dir_hardlinks);
	    }
	}

	return NMF_SUCC;

error:
	return ret;
}

static n_code doMount(n_dev* dev) {
	n_code ret;

	n_u32 const blk_size = dev->_param._bytes_per_block;
	dev->_blocks = nliAllocRaw(sizeof(n_block) * dev->_param._block_per_total);
	if (dev->_blocks == NULL)
		return NMF_FAIL(NMF_NOMEM);
	for (n_u32 i = 0; i < dev->_param._block_per_total; ++i) {
		n_block* block = &dev->_blocks[i];
		block->_offset = i * blk_size;
		block->_free_size = blk_size;
		block->_dirty_size = 0;
		INIT_LIST_HEAD(&block->__list);
	}

	ret = dev->_vtable._open(dev);
	if (nmfIsError(ret)) {
		return ret;
	}

	ret = buildFileSystem(dev);
	if (nmfIsError(ret)) {
		return ret;
	}

	return NMF_SUCC;
}

n_code nffsMount(n_dev** pdev, n_dev_param const* param, n_dev_vtable const* vtable, n_handle os_ctx) {
	n_dev* dev;
	n_code ret;
	dev = nliAllocDevice();
	if (dev == NULL)
		return NMF_FAIL(NMF_NOMEM);

	dev->_param = *param;
	dev->_vtable = *vtable;
	dev->_os_ctx = os_ctx;
	dev->_free_size = param->_bytes_per_total;

	ret = doMount(dev);
	if (nmfIsError(ret)) {
		nliFreeDevice(dev);
		return NMF_FAIL(NMF_NOMEM);
	}

	*pdev = dev;
	return NMF_SUCC;
}