/*
 * nffs_ext.h
 *
 *  Created on: 2021/5/24
 *      Author: ueyudiud
 */

#ifndef NFFS_EXT_H_
#define NFFS_EXT_H_

#include "nffs.h"

typedef struct nffsRichNodeRef n_rich_node_ref;
typedef struct nffsNodeFrag n_node_frag;

/**
 * 富节点引用，包含了节点的额外数据，仅在节点被实例化（外部文件引用）时产生
 */
struct nffsRichNodeRef {
	n_raw_node_ref*			_raw;
	n_u32					_off;			/* 节点的数据偏移量 */
	n_u32					_len;			/* 节点的数据长度 */
};

/**
 * 文件碎片
 */
struct nffsNodeFrag {
	n_rich_node_ref*		_ref;
	n_u32					_off;
	n_u32					_len;
	/* 不稳定的结构体，尽量避免直接访问以下字段 */
	n_node_frag*			_prev;
	n_node_frag*			_next;
};

#define frag_prev(f) ((f)->_prev)
#define frag_next(f) ((f)->_next)

struct nffsFile {
	n_inode*				_in;
	/* 不稳定的结构体，尽量避免直接访问以下字段 */
	n_dev*					_dev;
	n_node_frag*			_head;
	n_node_frag*			_tail;
};

#define frag_head(f) ((f)->_head)
#define frag_tail(f) ((f)->_tail)

#define nliGetDevFromFile(fp) ((fp)->_dev)

/* nodelist.c */
n_node_frag* nffsLookupNode(n_file* fp, n_u32 off);

#endif //NFFS_EXT_H_
